@@ -1,5 +1,6 @@
 /*
- * auth_mysql.c: authenticate users against a MySQL database
+ * auth_mysql.c:
+ * Authenticate users against a MySQL database.
  *
  * The only subtlety here is that the config directives for the database
  * (password etc.) are privileged information, which must be cleared prior to
@@ -16,7 +17,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_MYSQL
-static const char rcsid[] = "$Id: auth_mysql.c,v 1.24 2001/07/24 00:08:10 chris Exp $";
+static const char rcsid[] = "$Id: auth_mysql.c,v 1.34 2001/10/27 14:41:10 chris Exp $";
 
 #include <sys/types.h> /* BSD needs this here, apparently. */
 
@@ -39,6 +40,30 @@ static const char rcsid[] = "$Id: auth_mysql.c,v 1.24 2001/07/24 00:08:10 chris
 #include "stringmap.h"
 #include "util.h"
 
+/* Default query templates. The returned fields are:
+ *  [0] location of mailbox
+ *  [1] password hash
+ *  [2] unix user
+ *  [3] mailbox type
+ */
+char *user_pass_query_template =
+    "SELECT concat(domain.path, '/', popbox.mbox_name), popbox.password_hash, "
+            "domain.unix_user, 'bsd' "
+      "FROM popbox, domain "
+     "WHERE popbox.local_part = '$(local_part)' "
+       "AND popbox.domain_name = '$(domain)' "
+       "AND popbox.domain_name = domain.domain_name";
+       
+char *apop_query_template =
+    "SELECT concat(domain.path, '/', popbox.mbox_name), popbox.password_hash, "
+            "domain.unix_user, 'bsd' "
+      "FROM popbox, domain "
+     "WHERE popbox.local_part = '$(local_part)' "
+       "AND popbox.domain_name = '$(domain)' "
+       "AND popbox.domain_name = domain.domain_name";
+
+static char *substitute_query_params(const char *temp, const char *local_part, const char *domain);
+
 /* MD5 crypt(3) routines. This is here so that you can migrate passwords from
  * the modern /etc/shadow crypt_md5 format used (optionally) by recent
  * Linux-PAM distributions. This code was taken from Linux-PAM 0.75.
@@ -51,12 +76,10 @@ static const char rcsid[] = "$Id: auth_mysql.c,v 1.24 2001/07/24 00:08:10 chris
  *
  * Original author's notice:
  * 
- * ----------------------------------------------------------------------------
  * "THE BEER-WARE LICENSE" (Revision 42):
  * <phk@login.dknet.dk> wrote this file.  As long as you retain this notice you
  * can do whatever you want with this stuff. If we meet some day, and you think
  * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
- * ----------------------------------------------------------------------------
  */
 
 static unsigned char itoa64[] = /* 0 ... 63 => ascii - 64 */
@@ -182,12 +205,58 @@ static char *crypt_md5(const char *pw, const char *salt)
 
 /* MD5 crypt(3) routines end. */
 
+
+/* MySQL PASSWORD() routines. This is here so that you can use the MySQL
+ * proprietary password-hashing routine with tpop3d. The code is inserted here
+ * to avoid having to do an explicit query to get the MySQL password hash.
+ * Observe that this is not completely safe, since the machine on which the
+ * MySQL server is running may use a different character set to this machine.
+ * However, it is probably not worth worrying about this in reality.
+ *
+ * In fact, these functions will probably be available in libmysqlclient, but
+ * that doesn't appear to be documented, so better safe than sorry.
+ *
+ * This code is taken from the MySQL distribution. The original license for
+ * the code in sql/password.c states:
+ *
+ * Copyright Abandoned 1996 TCX DataKonsult AB & Monty Program KB & Detron HB
+ * This file is public domain and comes with NO WARRANTY of any kind
+ */
+
+/* hash_password:
+ * MySQL password-hashing routine. */
+static void mysql_hash_password(unsigned long *result, const char *password) {
+    register unsigned long nr=1345345333L, add=7, nr2=0x12345671L;
+    unsigned long tmp;
+    for (; *password; password++) {
+        if (*password == ' ' || *password == '\t')
+            continue;			/* skip space in password */
+        tmp  = (unsigned long) (unsigned char) *password;
+        nr  ^= (((nr & 63) + add) * tmp) + (nr << 8);
+        nr2 += (nr2 << 8) ^ nr;
+        add += tmp;
+    }
+    result[0] =  nr & (((unsigned long) 1L << 31) -1L); /* Don't use sign bit (str2int) */;
+    result[1] = nr2 & (((unsigned long) 1L << 31) -1L);
+    return;
+}
+
+/* make_scrambled_password:
+ * MySQL function to form a password hash and turn it into a string. */
+static void mysql_make_scrambled_password(char *to, const char *password) {
+    unsigned long hash_res[2];
+    mysql_hash_password(hash_res, password);
+    sprintf(to, "%08lx%08lx", hash_res[0], hash_res[1]);
+}
+
+/* MySQL PASSWORD() routines end. */
+
+
 /* strclr:
  * Clear a string.
  */
 static void strclr(char *s) {
-    char *p = s;
-    while (*p) *p++ = 0;
+    memset(s, 0, strlen(s));
 }
 
 /* auth_mysql_init:
@@ -211,8 +280,8 @@ int auth_mysql_init() {
 
     if ((I = stringmap_find(config, "auth-mysql-password"))) password = (char*)I->v;
     else {
-        print_log(LOG_ERR, _("auth_mysql_init: no auth-mysql-password directive in config"));
-        goto fail;
+        print_log(LOG_WARNING, _("auth_mysql_init: no auth-mysql-password directive in config; using blank password"));
+        password = "";
     }
 
     if ((I = stringmap_find(config, "auth-mysql-database"))) database = (char*)I->v;
@@ -224,6 +293,9 @@ int auth_mysql_init() {
     if ((I = stringmap_find(config, "auth-mysql-hostname"))) hostname = (char*)I->v;
     else hostname = localhost;
 
+    if ((I = stringmap_find(config, "auth-mysql-pass-query"))) user_pass_query_template = (char*)I->v;
+    if ((I = stringmap_find(config, "auth-mysql-apop-query"))) apop_query_template = (char*)I->v;
+
     mysql = mysql_init(NULL);
     if (!mysql) {
         print_log(LOG_ERR, _("auth_mysql_init: mysql_init: failed"));
@@ -248,29 +320,19 @@ fail:
     return ret;
 }
 
+extern int verbose; /* in main.c */
+
 /* auth_mysql_new_apop:
- * Attempt to authenticate a user via APOP, using a SELECT statement of the
- * form
- *   SELECT domain.path, popbox.mbox_name, popbox.pwhash, domain.unix_user
- *           FROM popbox, domain
- *          WHERE popbox.local_part = $local_part
- *            AND popbox.domain_name = $domain
- *            AND popbox.domain_name = domain.domain_name
+ * Attempt to authenticate a user via APOP, using the template SELECT query in
+ * the config file or the default defined above otherwise.
  */
-const char apop_query_template[] =
-    "SELECT domain.path, popbox.mbox_name, popbox.password_hash, domain.unix_user "
-      "FROM popbox, domain "
-     "WHERE popbox.local_part = '%s' "
-       "AND popbox.domain_name = '%s' "
-       "AND popbox.domain_name = domain.domain_name";
-authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const unsigned char *digest) {
-    char *query = NULL, *x = NULL, *y = NULL;
+authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const unsigned char *digest, const char *host /* unused */) {
+    char *query = NULL;
     authcontext a = NULL;
     char *local_part = NULL;
     const char *domain;
     item *I;
     int use_gid = 0;
-    size_t l;
     gid_t gid;
 
     if (!mysql) return NULL;
@@ -288,7 +350,7 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
     if (domain == name || !*domain) return NULL;
     ++domain;
     
-    local_part = (char*)malloc(domain - name);
+    local_part = xmalloc(domain - name);
     if (!local_part) return NULL;
     memset(local_part, 0, domain - name);
     strncpy(local_part, name, domain - name - 1);
@@ -298,15 +360,12 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
         return NULL;
     }
 
-    query = (char*)malloc(l = (sizeof(apop_query_template) + strlen(name) * 2 + 1));
-    x = (char*)malloc(strlen(local_part) * 2 + 1);
-    y = (char*)malloc(strlen(domain) * 2 + 1);
-    if (!query || !x || !y) goto fail;
+    /* Obtain the actual query to use. */
+    query = substitute_query_params(apop_query_template, local_part, domain);
+    if (!query) goto fail;
 
-    mysql_escape_string(x, local_part, strlen(local_part));
-    mysql_escape_string(y, domain, strlen(domain));
-
-    snprintf(query, l, apop_query_template, x, y);
+    if (verbose)
+        print_log(LOG_DEBUG, "auth_mysql_new_apop: SQL query: %s", query);
 
     if (mysql_query(mysql, query) == 0) {
         MYSQL_RES *result = mysql_store_result(mysql);
@@ -317,6 +376,11 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
             goto fail;
         }
 
+        if (mysql_field_count(mysql) != 4) {
+            print_log(LOG_ERR, "auth_mysql_new_apop: %d fields returned by query, should be 4: mailbox location, password hash, unix user, mailbox type", mysql_field_count(mysql));
+            goto fail;
+        }
+
         switch (i = mysql_num_rows(result)) {
         case 0:
             break;
@@ -325,7 +389,6 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
                 unsigned long *lengths;
                 struct passwd *pw;
                 unsigned char this_digest[16];
-                char *mailbox;
                 MD5_CTX ctx;
                 uid_t uid;
 
@@ -333,7 +396,7 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
                 if (!row || !(lengths = mysql_fetch_lengths(result))) break;
 
                 /* Verify that this user has a plaintext password. */
-                if (strncmp(row[2], "{plaintext}", 11) != 0) {
+                if (strncmp(row[1], "{plaintext}", 11) != 0) {
                     print_log(LOG_WARNING, _("auth_mysql_new_apop: attempted APOP login by %s@%s, who does not have a plaintext password"), local_part, domain);
                     break;
                 }
@@ -341,7 +404,7 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
                 /* Calculate our idea of the digest */
                 MD5Init(&ctx);
                 MD5Update(&ctx, (unsigned char*)timestamp, strlen(timestamp));
-                MD5Update(&ctx, (unsigned char*)row[2] + 11, lengths[2] - 11);
+                MD5Update(&ctx, (unsigned char*)row[1] + 11, lengths[1] - 11);
                 MD5Final(this_digest, &ctx);
 
                 /* User was lying */
@@ -351,7 +414,7 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
                 }
 
                 /* User was not lying (about her password) */
-                if (!parse_uid((const char*)row[3], &uid)) {
+                if (!parse_uid((const char*)row[2], &uid)) {
                     print_log(LOG_ERR, _("auth_mysql_new_apop: unix user `%s' for %s@%s does not make sense"), row[3], local_part, domain);
                     break;
                 }
@@ -366,11 +429,8 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
                 /* We will let the program figure out which sort of mailbox
                  * to use later.
                  */
-                mailbox = (char*)malloc(lengths[0] + lengths[1] + 2);
-                sprintf(mailbox, "%s/%s", row[0], row[1]);
                 a = authcontext_new(pw->pw_uid, use_gid ? gid : pw->pw_gid,
-                                    NULL, mailbox, NULL, domain); /* note default mailbox type */
-                free(mailbox);
+                                    row[3], row[0], pw->pw_dir, domain);
 
                 break;
             }
@@ -387,38 +447,23 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
     }
 
 fail:
-    if (local_part) free(local_part);
-    if (x) free(x);
-    if (y) free(y);
-    if (query) free(query);
+    if (local_part) xfree(local_part);
+    if (query) xfree(query);
 
     return a;
 }
 
 /* auth_mysql_new_user_pass:
- * Attempt to authenticate a user via USER/PASS, using a SELECT statement of
- * the form
- *   SELECT domain.path, popbox.mbox_name, popbox.password_hash, domain.unix_user
- *          FROM popbox, domain
- *          WHERE popbox.local_part = $local_part
- *            AND popbox.domain_name = $domain
- *            AND popbox.domain_name = domain.domain_name
- */
-char user_pass_query_template[] =
-    "SELECT domain.path, popbox.mbox_name, popbox.password_hash, domain.unix_user "
-      "FROM popbox, domain "
-     "WHERE popbox.local_part = '%s' "
-       "AND popbox.domain_name = '%s' "
-       "AND popbox.domain_name = domain.domain_name";
-authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
-    char *query = NULL, *x = NULL, *y = NULL;
+ * Attempt to authenticate a user via USER/PASS, using the template SELECT
+ * query in the config file or the default defined above otherwise. */
+authcontext auth_mysql_new_user_pass(const char *user, const char *pass, const char *host /* unused */) {
+    char *query = NULL;
     authcontext a = NULL;
     char *local_part = NULL;
     const char *domain;
     char *p;
     unsigned char *q;
     item *I;
-    size_t l;
     int use_gid = 0;
     gid_t gid;
 
@@ -437,7 +482,7 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
     if (domain == user || !*domain) return NULL;
     ++domain;
     
-    local_part = (char*)malloc(domain - user);
+    local_part = xmalloc(domain - user);
     if (!local_part) return NULL;
     memset(local_part, 0, domain - user);
     strncpy(local_part, user, domain - user - 1);
@@ -447,15 +492,12 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
         return NULL;
     }
 
-    query = (char*)malloc(l = (sizeof(user_pass_query_template) + strlen(user) * 2));
-    x = (char*)malloc(strlen(local_part) * 2 + 1);
-    y = (char*)malloc(strlen(domain) * 2 + 1);
-    if (!query || !x || !y) goto fail;
+    /* Obtain the actual query to use. */
+    query = substitute_query_params(user_pass_query_template, local_part, domain);
+    if (!query) goto fail;
 
-    mysql_escape_string(x, local_part, strlen(local_part));
-    mysql_escape_string(y, domain, strlen(domain));
-
-    snprintf(query, l, user_pass_query_template, x, y);
+    if (verbose)
+        print_log(LOG_DEBUG, "auth_mysql_new_user_pass: SQL query: %s", query);
 
     if (mysql_query(mysql, query) == 0) {
         MYSQL_RES *result = mysql_store_result(mysql);
@@ -466,6 +508,11 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
             goto fail;
         }
 
+        if (mysql_field_count(mysql) != 4) {
+            print_log(LOG_ERR, "auth_mysql_new_user_pass: %d fields returned by query, should be 4: mailbox location, password hash, unix user, mailbox type", mysql_field_count(mysql));
+            goto fail;
+        }
+
         switch (i = mysql_num_rows(result)) {
         case 0:
             break;
@@ -473,7 +520,6 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
                 MYSQL_ROW row = mysql_fetch_row(result);
                 unsigned long *lengths;
                 char *pwhash;
-                char *mailbox;
                 struct passwd *pw;
                 int authok = 0;
                 uid_t uid;
@@ -482,7 +528,7 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
                 if (!row || !(lengths = mysql_fetch_lengths(result))) break;
 
                 /* Verify the password. There are several possibilities here. */
-                pwhash = (char*)row[2];
+                pwhash = (char*)row[1];
 
                 if (strncmp(pwhash, "{crypt}", 7) == 0) {
                     /* Password hashed by system crypt function. */
@@ -493,6 +539,26 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
                 } else if (strncmp(pwhash, "{plaintext}", 11) == 0) {
                     /* Plain text password, as used for APOP. */
                     if (strcmp(pass, pwhash + 11) == 0) authok = 1;
+                } else if (strncmp(pwhash, "{mysql}", 7) == 0) {
+                    /* MySQL PASSWORD() type password hash. */
+                    char hash[17] = {0};
+                    int n;
+                    mysql_make_scrambled_password(hash, pass);
+                    /* The MySQL password format changed, and we should accept
+                     * either a 16- or 8-character long hash. */
+                    switch (n = strlen(pwhash + 7)) {
+                        case 8:
+                            if (strncmp(pwhash + 7, hash, 8) == 0) authok = 1;
+                            break;
+
+                        case 16:
+                            if (strcmp(pwhash + 7, hash) == 0) authok = 1;
+                            break;
+
+                        default:
+                            print_log(LOG_ERR, _("auth_mysql_new_user_pass: %s@%s has mysql password type, but hash is of incorrect length %d"), local_part, domain, n);
+                            break;
+                    }
                 } else if (strncmp(pwhash, "{md5}", 4) == 0 || *pwhash != '{') {
                     /* Straight MD5 password. */
                     MD5_CTX ctx;
@@ -516,7 +582,7 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
                     break;
                 }
 
-                if (!parse_uid((const char*)row[3], &uid)) {
+                if (!parse_uid((const char*)row[2], &uid)) {
                     print_log(LOG_ERR, _("auth_mysql_new_user_pass: unix user `%s' for %s@%s does not make sense"), row[3], local_part, domain);
                     break;
                 }
@@ -528,12 +594,8 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
                     break;
                 }
 
-                mailbox = (char*)malloc(l = (lengths[0] + lengths[1] + 2));
-                snprintf(mailbox, l, "%s/%s", row[0], row[1]);
                 a = authcontext_new(pw->pw_uid, use_gid ? gid : pw->pw_gid,
-                                    NULL, mailbox, NULL, domain); /* note default mailbox type. */
-                free(mailbox);
-
+                                    row[3], row[0], pw->pw_dir, domain);
                 break;
             }
 
@@ -548,10 +610,8 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
     }
 
 fail:
-    if (local_part) free(local_part);
-    if (x) free(x);
-    if (y) free(y);
-    if (query) free(query);
+    if (local_part) xfree(local_part);
+    if (query) xfree(query);
 
     return a;
 }
@@ -563,4 +623,33 @@ void auth_mysql_close() {
     if (mysql) mysql_close(mysql);
 }
 
+/* substitute_query_params
+ * Given a query template, a localpart and a domain, return a copy of the
+ * template with the fields filled in.
+ */
+static char *substitute_query_params(const char *template, const char *local_part, const char *domain) {
+    char *query, *l, *d;
+    struct sverr err;
+
+    /* Form escaped copies of the user and domain. */
+    if (!(l = xmalloc(strlen(local_part) * 2 + 1)))
+	return NULL;
+    mysql_escape_string(l, local_part, strlen(local_part));
+
+    if (!(d = xmalloc(strlen(domain) * 2 + 1))) {
+	xfree(l);
+	return NULL;
+    }
+    mysql_escape_string(d, domain, strlen(domain));
+
+    /* Do the substitution. */
+    query = substitute_variables(template, &err, 2, "local_part", l, "domain", d);
+    if (!query)
+        print_log(LOG_ERR, _("substitute_query_params: %s near `%.16s'"), err.msg, template + err.offset);
+    
+    xfree(l);
+    xfree(d);
+    return query;
+}
+
 #endif /* AUTH_MYSQL */
