@@ -14,6 +14,7 @@
  *  user        client's username sent with USER or APOP command
  *  pass        client's password from PASS command
  *  digest      client's digest sent with APOP command, in hex
+ *  clienthost  client's IP address/hostname
  *
  * The program should respond with a similarly formatted string containing the
  * following variables:
@@ -43,7 +44,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_OTHER
-static const char rcsid[] = "$Id: auth_other.c,v 1.8 2001/07/20 11:47:28 chris Exp $";
+static const char rcsid[] = "$Id: auth_other.c,v 1.14 2001/10/18 10:11:29 chris Exp $";
 
 #include <sys/types.h>
 
@@ -78,7 +79,7 @@ volatile int authchild_status;
 struct timeval authchild_timeout;
 
 /* File descriptors used to talk to child. */
-volatile int authchild_wr, authchild_rd;
+volatile int authchild_wr = -1, authchild_rd = -1;
 
 /* dump:
  * Debugging method.
@@ -86,7 +87,7 @@ volatile int authchild_wr, authchild_rd;
 void dump(unsigned char *b, size_t len) {
     unsigned char *p;
     char *q;
-    char *str = malloc(len * 4 + 1);
+    char *str = xmalloc(len * 4 + 1);
     for (p = b, q = str; p < b + len; ++p)
         if (*p >= 32 && *p <= 127) *q++ = *p;
         else {
@@ -95,7 +96,7 @@ void dump(unsigned char *b, size_t len) {
         }
     *q = 0;
     print_log(LOG_INFO, "dump %s", str);
-    free(str);
+    xfree(str);
 }
 
 /* tvadd:
@@ -445,7 +446,7 @@ stringmap auth_other_recv_response() {
      * value.
      */
     q = buffer;
-    while (*q) {
+    while (*q && S) {
         r = memchr(q, 0, p - q);
         if (!r || r > (p - 3)) goto formaterror;
         
@@ -471,7 +472,7 @@ fail:
 /* auth_other_new_apop:
  * Attempt to authenticate a user using APOP, via the child program.
  */
-authcontext auth_other_new_apop(const char *name, const char *timestamp, const unsigned char *digest) {
+authcontext auth_other_new_apop(const char *name, const char *timestamp, const unsigned char *digest, const char *host) {
 #define MISSING(k)     do { print_log(LOG_ERR, _("auth_other_new_apop: missing key `%s' in response"), (k)); goto fail; } while(0)
 #define INVALID(k, v)  do { print_log(LOG_ERR, _("auth_other_new_apop: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
     char digeststr[33];
@@ -485,7 +486,7 @@ authcontext auth_other_new_apop(const char *name, const char *timestamp, const u
     
     for (p = digeststr, q = digest; q < digest + 16; p += 2, ++q)
         sprintf(p, "%02x", (unsigned int)*q);
-    if (!auth_other_send_request(4, "method", "APOP", "user", name, "timestamp", timestamp, "digest", digeststr)
+    if (!auth_other_send_request(5, "method", "APOP", "user", name, "timestamp", timestamp, "digest", digeststr, "clienthost", host)
         || !(S = auth_other_recv_response()))
         return NULL;
 
@@ -534,7 +535,7 @@ fail:
 /* auth_other_new_user_pass:
  * Attempt to authenticate a user using USER/PASS, via the child program.
  */
-authcontext auth_other_new_user_pass(const char *user, const char *pass) {
+authcontext auth_other_new_user_pass(const char *user, const char *pass, const char *host) {
 #define MISSING(k)     do { print_log(LOG_ERR, _("auth_other_new_user_pass: missing key `%s' in response"), (k)); goto fail; } while(0)
 #define INVALID(k, v)  do { print_log(LOG_ERR, _("auth_other_new_user_pass: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
     stringmap S;
@@ -543,7 +544,7 @@ authcontext auth_other_new_user_pass(const char *user, const char *pass) {
 
     if (!authchild_pid) auth_other_start_child();
 
-    if (!auth_other_send_request(3, "method", "PASS", "user", user, "pass", pass)
+    if (!auth_other_send_request(4, "method", "PASS", "user", user, "pass", pass, "clienthost", host)
         || !(S = auth_other_recv_response()))
         return NULL;
     
@@ -568,7 +569,7 @@ authcontext auth_other_new_user_pass(const char *user, const char *pass) {
        
         I = stringmap_find(S, "gid");
         if (!I) MISSING("gid");
-        else if (!parse_gid(I->v, &uid)) INVALID("gid", (char*)I->v);
+        else if (!parse_gid(I->v, &gid)) INVALID("gid", (char*)I->v);
 
         I = stringmap_find(S, "mailbox");
         if (I) mailbox = (char*)I->v;
