@@ -11,7 +11,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_PAM
-static const char rcsid[] = "$Id: auth_pam.c,v 1.15 2001/06/10 21:52:10 chris Exp $";
+static const char rcsid[] = "$Id: auth_pam.c,v 1.18 2001/10/14 01:13:37 chris Exp $";
 
 #include <sys/types.h> /* BSD needs this here, it seems. */
 
@@ -41,7 +41,7 @@ int auth_pam_conversation(int num_msg, const struct pam_message **msg, struct pa
 
     if (!num_msg || !msg || !appdata_ptr) return PAM_CONV_ERR;
     
-    *resp = (struct pam_response*)calloc(num_msg, sizeof(struct pam_response));
+    *resp = (struct pam_response*)xcalloc(num_msg, sizeof(struct pam_response));
     if (!*resp) return PAM_CONV_ERR;
 
     /* Assume that any prompt is asking for a password */
@@ -60,7 +60,7 @@ int auth_pam_conversation(int num_msg, const struct pam_message **msg, struct pa
  */
 extern stringmap config;
 
-authcontext auth_pam_new_user_pass(const char *user, const char *pass) {
+authcontext auth_pam_new_user_pass(const char *user, const char *pass, const char *host /* unused */) {
     pam_handle_t *pamh = NULL;
     struct passwd pw, *pw2;
     int r, n = PAM_SUCCESS;
