@@ -11,7 +11,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_PASSWD
-static const char rcsid[] = "$Id: auth_passwd.c,v 1.11 2001/07/24 00:08:10 chris Exp $";
+static const char rcsid[] = "$Id: auth_passwd.c,v 1.13 2001/10/14 02:09:55 chris Exp $";
 
 #include <sys/types.h>
 
@@ -45,12 +45,11 @@ static const char rcsid[] = "$Id: auth_passwd.c,v 1.11 2001/07/24 00:08:10 chris
  */
 extern stringmap config;
 
-authcontext auth_passwd_new_user_pass(const char *user, const char *pass) {
+authcontext auth_passwd_new_user_pass(const char *user, const char *pass, const char *host /* unused */) {
     struct passwd *pw;
 #ifdef AUTH_PASSWD_SHADOW
     struct spwd *spw;
 #endif /* AUTH_PASSWD_SHADOW */
-    char *mailspool_dir;
     char *user_passwd;
     item *I;
     int use_gid = 0;
