@@ -7,7 +7,7 @@
  */
 
 static const char copyright[] = "$Copyright: (c) 2001 Chris Lightfoot. All rights reserved. $";
-static const char rcsid[] = "$Id: main.c,v 1.52 2001/07/24 00:08:10 chris Exp $";
+static const char rcsid[] = "$Id: main.c,v 1.59 2001/10/23 15:07:44 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -23,6 +23,12 @@ static const char rcsid[] = "$Id: main.c,v 1.52 2001/07/24 00:08:10 chris Exp $"
 #include <time.h>
 #include <unistd.h>
 
+#ifdef USE_TCP_WRAPPERS
+#include <tcpd.h>
+int allow_severity = LOG_INFO;
+int deny_severity  = LOG_NOTICE;
+#endif
+
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
@@ -34,7 +40,6 @@ static const char rcsid[] = "$Id: main.c,v 1.52 2001/07/24 00:08:10 chris Exp $"
 
 #include "config.h"
 #include "connection.h"
-#include "errprintf.h"
 #include "listener.h"
 #include "pidfile.h"
 #include "signals.h"
@@ -64,6 +69,8 @@ volatile int num_running_children = 0;  /* How many children are active. */
 
 char *pidfile = NULL;               /* The name of a PID file to use; if NULL, don't use one. */
 
+char *tcpwrappersname = "tpop3d";   /* The daemon name to give to TCP Wrappers. */
+
 /* Variables representing the state of the server. */
 int post_fork = 0;                  /* Is this a child handling a connection. */
 connection this_child_connection;   /* Stored here so that if a signal terminates the child, the mailspool will still get unlocked. */
@@ -127,7 +134,14 @@ void listeners_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfds)
 
             if (s == -1) {
                 if (errno != EAGAIN) print_log(LOG_ERR, "net_loop: accept: %m");
-            } else if (setsockopt(s, SOL_SOCKET, SO_SNDBUF, &a, sizeof(a)) == -1) {
+            }
+#ifdef USE_TCP_WRAPPERS
+            else if (!hosts_ctl(tcpwrappersname, STRING_UNKNOWN, inet_ntoa(sin.sin_addr), STRING_UNKNOWN)) {
+                print_log(LOG_ERR, "net_loop: tcp_wrappers: connection from %s refused", inet_ntoa(sin.sin_addr));
+                close(s);
+            }
+#endif
+            else if (setsockopt(s, SOL_SOCKET, SO_SNDBUF, &a, sizeof(a)) == -1) {
                 /* Set a small send buffer so that we get usefully blocking
                  * writes. */
                 print_log(LOG_ERR, "listeners_post_select: setsockopt: %m");
@@ -172,8 +186,12 @@ void connections_pre_select(int *n, fd_set *readfds, fd_set *writefds, fd_set *e
 }
 
 /* fork_child:
- * Handle forking a child to handle an individual connection c.
+ * Handle forking a child to handle an individual connection c after authentication.
  */
+#ifdef AUTH_OTHER
+extern volatile int authchild_wr, authchild_rd; /* in auth_other.c */
+#endif /* AUTH_OTHER */
+
 void fork_child(connection c) {
     connection *J;
     item *t;
@@ -187,7 +205,9 @@ void fork_child(connection c) {
     sigprocmask(SIG_BLOCK, &chmask, NULL);
 
     post_fork = 1; /* This is right. See below. */
-    /* muntrace(); */ /* Memory debugging on glibc systems. */
+#ifdef MTRACE_DEBUGGING
+    muntrace(); /* Memory debugging on glibc systems. */
+#endif /* MTRACE_DEBUGGING */
     switch((ch = fork())) {
         case 0:
             /* Child. Dispose of listeners and connections other than this
@@ -204,6 +224,12 @@ void fork_child(connection c) {
                     connection_delete(*J);
                     *J = NULL;
                 }
+            
+#ifdef AUTH_OTHER
+            /* Need to close the pipe to the authentication child. */
+            if (authchild_rd != -1) close(authchild_rd);
+            if (authchild_wr != -1) close(authchild_wr);
+#endif /* AUTH_OTHER */
 
             /* We never access mailspools as root. */
             if (c->a->uid == 0) {
@@ -322,6 +348,7 @@ void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfd
                         pop3command_delete(p);
                         switch (act) {
                             case close_connection:
+                                print_log(LOG_INFO, _("connections_post_select: client %s: disconnected"), c->idstr);
                                 remove_connection(c);
                                 connection_delete(c);
                                 *I = c = NULL;
@@ -381,7 +408,7 @@ void net_loop() {
 
     /* 2 * max_running_children is a reasonable ball-park figure. */
     max_connections = 2 * max_running_children;
-    connections = (connection*)calloc(max_connections, sizeof(connection*));
+    connections = (connection*)xcalloc(max_connections, sizeof(connection*));
 
     print_log(LOG_INFO, _("net_loop: tpop3d version %s successfully started"), TPOP3D_VERSION);
     
@@ -432,7 +459,7 @@ void net_loop() {
     if (connections) {
         for (J = connections; J < connections + max_connections; ++J)
             if (*J) connection_delete(*J);
-        free(connections);
+        xfree(connections);
     }
 }
 
@@ -459,6 +486,12 @@ void usage(FILE *fp) {
     authswitch_describe(fp);
     mailbox_describe(fp);
 
+#ifdef USE_TCP_WRAPPERS
+    fprintf(fp, _("This tpop3d has TCP Wrappers support.\n\n"));
+#else
+    fprintf(fp, _("This tpop3d does not have TCP Wrappers support.\n\n"));
+#endif
+    
     fprintf(fp, _(
 "tpop3d, copyright (c) 2000-2001 Chris Lightfoot <chris@ex-parrot.com>;\n"
 "portions copyright (c) 2001 Mark Longair, Paul Makepeace.\n"
@@ -484,11 +517,13 @@ extern int mailspool_save_indices;  /* in mailspool.c */
 int main(int argc, char **argv, char **envp) {
     item *I;
     int nodaemon = 0;
-    char *configfile = "/etc/tpop3d.conf", c;
-    int na;
-
-    /*mtrace();*/ /* Memory debugging on glibc systems. */
+    char *configfile = "/etc/tpop3d.conf";
+    int na, c;
 
+#ifdef MTRACE_DEBUGGING
+    mtrace(); /* Memory debugging on glibc systems. */
+#endif /* MTRACE_DEBUGGING */
+    
     /* Read the options. */
     opterr = 0;
     while ((c = getopt(argc, argv, optstring)) != EOF) {
@@ -540,12 +575,21 @@ int main(int argc, char **argv, char **envp) {
     I = stringmap_find(config, "mailspool-index");
     if (I) {
         mailspool_save_indices = 1;
-        print_log(LOG_INFO, "experimental BSD mailbox metadata cache enabled");
+        print_log(LOG_INFO, _("experimental BSD mailbox metadata cache enabled"));
     }
 #endif
+
+    /* We may have been compiled with TCP wrappers support. */
+#ifdef USE_TCP_WRAPPERS
+    I = stringmap_find(config, "tcp-wrappers-name");
+    if (I)
+        tcpwrappersname = I->v;
+    print_log(LOG_INFO, _("TCP Wrappers support enabled, using daemon name `%s'"), tcpwrappersname);
+#endif
     
     /* Try to write PID file. */
     if (pidfile) {
+retry_pid_file:
         switch (write_pid_file(pidfile)) {
             case pid_file_success:
                 break;
@@ -555,10 +599,14 @@ int main(int argc, char **argv, char **envp) {
                 switch (read_pid_file(pidfile, &pid)) {
                     case pid_file_success:
                         if (kill(pid, 0)) {
-                            print_log(LOG_ERR, _("%s: stale PID file `%s'; exiting. Remove it and restart."), pidfile);
-                            return 1;
+                            print_log(LOG_ERR, _("%s: stale PID file; removing it"), pidfile);
+                            if (unlink(pidfile) == -1) {
+                                print_log(LOG_ERR, _("%s: stale PID file: unlink: %m"), pidfile);
+                                return 1;
+                            } else goto retry_pid_file; /* harmful? */
+                            
                         } else {
-                            print_log(LOG_ERR, _("%s: tpop3d already running, with process ID %d; exiting."), (int)pid);
+                            print_log(LOG_ERR, _("%s: tpop3d already running, with process ID %d; exiting"), (int)pid);
                             return 1;
                         }
                         break;
@@ -701,6 +749,8 @@ int main(int argc, char **argv, char **envp) {
         vector_delete(listeners);
     }
 
+    stringmap_delete_free(config);
+    
     /* We may have got here because we're supposed to terminate and restart. */
     if (restart) {
         execve(argv[0], argv, envp);
@@ -710,36 +760,3 @@ int main(int argc, char **argv, char **envp) {
     EXIT_REMOVING_PIDFILE(0);
 }
 
-#if 0
-/* Primitive memory-leak debugging. */
-#undef malloc
-#undef free
-#undef realloc
-
-char *mystrdup(char *f, int l, const char *s) {
-    char *p = malloc(strlen(s) + 1);
-    strcpy(p, s);
-    fprintf(stderr, "[%d] %s:%d: %p = strdup(\"%s\")\n", getpid(), f, l, p, s);
-    return p;
-}
-
-void *mymalloc(char *f, int l, const size_t n) {
-    void *p = malloc(n);
-    if (!p) return NULL;
-    fprintf(stderr, "[%d] %s:%d: %p = malloc(%d)\n", getpid(), f, l, p, n);
-    return p;
-}
-
-void myfree(char *f, int l, void *p) {
-    free(p);
-    fprintf(stderr, "[%d] %s:%d: free(%p)\n", getpid(), f, l, p);
-}
-
-void *myrealloc(char *f, int l, void *p, const size_t n) {
-    void *q = realloc(p, n);
-    fprintf(stderr, "[%d] %s:%d: free(%p)          (actually realloc)\n", getpid(), f, l, p);
-    fprintf(stderr, "[%d] %s:%d: malloc(%p, %d)    (actually realloc)\n", getpid(), f, l, p, n);
-    return q;
-}
-
-#endif 
