@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: pop3.c,v 1.31 2001/07/24 00:08:10 chris Exp $";
+static const char rcsid[] = "$Id: pop3.c,v 1.34 2001/10/14 01:13:38 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -38,7 +38,7 @@ int append_domain;  /* Do we automatically try user@domain if user alone fails t
 enum connection_action connection_do(connection c, const pop3command p) {
     /* This breaks the RFC, but is sensible. */
     if (p->cmd != NOOP && p->cmd != UNKNOWN) c->idlesince = time(NULL);
-    
+
     if (c->state == authorisation) {
         /* Authorisation state: gather username and password. */
         switch (p->cmd) {
@@ -83,6 +83,8 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 char *name, *hexdigest;
                 unsigned char digest[16];
 
+                ++c->n_auth_tries;
+
                 if (p->toks->num != 3) {
                     connection_sendresponse(c, 0, _("No, that's not right."));
                     return do_nothing;
@@ -100,8 +102,6 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     return close_connection;
                 }
 
-                ++c->n_auth_tries;
-
                 if (!name || *name == 0) {
                     connection_sendresponse(c, 0, _("That's not right."));
                 }
@@ -125,22 +125,22 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     return do_nothing;
                 }
 
-                c->a = authcontext_new_apop(name, c->timestamp, digest, c->domain);
+                c->a = authcontext_new_apop(name, c->timestamp, digest, c->domain, inet_ntoa(c->sin.sin_addr));
                 
                 if (!c->a && append_domain && c->domain && strcspn(name, "@%!") == strlen(name)) {
                     /* OK, if we have a domain name, try appending that. */
-                    char *nn = (char*)malloc(strlen(name) + strlen(c->domain) + 2);
+                    char *nn = xmalloc(strlen(name) + strlen(c->domain) + 2);
                     strcpy(nn, name);
                     strcat(nn, "@");
                     strcat(nn, c->domain);
-                    c->a = authcontext_new_apop(nn, c->timestamp, digest, c->domain);
-                    free(nn);
+                    c->a = authcontext_new_apop(nn, c->timestamp, digest, c->domain, inet_ntoa(c->sin.sin_addr));
+                    xfree(nn);
                 }
 
                 if (c->a) {
                     /* Now save a new ID string for this client. */
-                    free(c->idstr);
-                    c->idstr =(char*)malloc(strlen(c->a->user) + 2 + strlen(inet_ntoa(c->sin.sin_addr)) + 16);
+                    xfree(c->idstr);
+                    c->idstr =xmalloc(strlen(c->a->user) + 2 + strlen(inet_ntoa(c->sin.sin_addr)) + 16);
                     sprintf(c->idstr, "[%d]%s(%s)", c->s, c->a->user, inet_ntoa(c->sin.sin_addr));
 
                     c->state = transaction;
@@ -156,7 +156,6 @@ enum connection_action connection_do(connection c, const pop3command p) {
                         print_log(LOG_ERR, _("connection_do: client `%s' failed to log in after %d attempts"), c->idstr, MAX_AUTH_TRIES);
                         return close_connection;
                     } else {
-                        ++c->n_auth_tries;
 #ifndef NO_SNIDE_COMMENTS
                         connection_sendresponse(c, 0, _("Lies! Try again!"));
 #else
@@ -192,31 +191,31 @@ enum connection_action connection_do(connection c, const pop3command p) {
 
         /* Do we now have enough information to authenticate using USER/PASS? */
         if (!c->a && c->user && c->pass) {
-            c->a = authcontext_new_user_pass(c->user, c->pass, c->domain);
+            c->a = authcontext_new_user_pass(c->user, c->pass, c->domain, inet_ntoa(c->sin.sin_addr));
             if (!c->a && append_domain && c->domain && strcspn(c->user, "@%!") == strlen(c->user)) {
                 /* OK, if we have a domain name, try appending that. */
-                char *nn = (char*)malloc(strlen(c->user) + strlen(c->domain) + 2);
+                char *nn = xmalloc(strlen(c->user) + strlen(c->domain) + 2);
                 strcpy(nn, c->user);
                 strcat(nn, "@");
                 strcat(nn, c->domain);
-                c->a = authcontext_new_user_pass(nn, c->pass, c->domain);
-                free(nn);
+                c->a = authcontext_new_user_pass(nn, c->pass, c->domain, inet_ntoa(c->sin.sin_addr));
+                xfree(nn);
             }
 
             if (c->a) {
                 /* Now save a new ID string for this client. */
-                free(c->idstr);
-                c->idstr =(char*)malloc(strlen(c->a->user) + 2 + strlen(inet_ntoa(c->sin.sin_addr)) + 16);
+                xfree(c->idstr);
+                c->idstr =xmalloc(strlen(c->a->user) + 2 + strlen(inet_ntoa(c->sin.sin_addr)) + 16);
                 sprintf(c->idstr, "[%d]%s(%s)", c->s, c->a->user, inet_ntoa(c->sin.sin_addr));
 
                 memset(c->pass, 0, strlen(c->pass));
                 c->state = transaction;
                 return fork_and_setuid; /* Code in main.c sends response in case of error. */
             } else {
-                free(c->user);
+                xfree(c->user);
                 c->user = NULL;
                 memset(c->pass, 0, strlen(c->pass));
-                free(c->pass);
+                xfree(c->pass);
                 c->pass = NULL;
 
                 ++c->n_auth_tries;
@@ -316,7 +315,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     connection_sendresponse(c, 0, _("That message is no more."));
                 else {
                     char response[32] = {0};
-                    snprintf(response, 31, "%d %d", 1 + msg_num, curmsg->msglength - curmsg->length - 1);
+                    snprintf(response, 31, "%d %d", 1 + msg_num, (int)(curmsg->msglength - curmsg->length - 1));
                     connection_sendresponse(c, 1, response);
                 }
             } else {
@@ -326,7 +325,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 for (m = curmbox->index; m < curmbox->index + curmbox->num; ++m) {
                     if (!m->deleted) {
                         char response[32] = {0};
-                        snprintf(response, 31, "%d %d", 1 + m - curmbox->index, m->msglength - m->length - 1);
+                        snprintf(response, 31, "%d %d", 1 + m - curmbox->index, (int)(m->msglength - m->length - 1));
                         connection_sendline(c, response);
                         ++nn;
                     }
