@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: util.c,v 1.8 2001/07/08 16:57:09 chris Exp $";
+static const char rcsid[] = "$Id: util.c,v 1.10 2001/10/14 01:13:38 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -42,7 +42,6 @@ void print_log(int priority, const char *fmt, ...) {
     va_end(ap);
     syslog(priority, "%s", s);
     if (log_stderr) fprintf(stderr, "%s\n", s);
-    free(s);
 }
 
 /* xwrite:
@@ -270,3 +269,36 @@ int unhex_digest(const char *from, unsigned char *to) {
 
     return 1;
 }
+
+/* xmalloc:
+ * Malloc, and abort if malloc fails. */
+void *xmalloc(size_t n) {
+    void *v;
+    v = malloc(n);
+    if (!v) abort();
+    return v;
+}
+
+/* xcalloc:
+ * As above. */
+void *xcalloc(size_t n, size_t m) {
+    void *v;
+    v = calloc(n, m);
+    if (!v) abort();
+    return v;
+}
+
+/* xrealloc:
+ * As above. */
+void *xrealloc(void *w, size_t n) {
+    void *v;
+    v = realloc(w, n);
+    if (n != 0 && !v) abort();
+    return v;
+}
+
+/* xfree:
+ * Free, ignoring a passed NULL value. */
+void xfree(void *v) {
+    if (v) free(v);
+}
